// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
// Last Modified: 8th August 2006
//
// Purpose:       Demo shows how to use hel_MapCreateIndirect
//
// -----------------------------------------------------------------------------
#include <hel2.h>
#include "ResourceData.h"

// -----------------------------------------------------------------------------
// Global Variables
// -----------------------------------------------------------------------------
u8 ATTR_EWRAM ATTR_ALIGNED(4) g_MapSystemBuffer[HEL_SUBSYSTEM_MAP_REQUIREDMEMORY];

// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void CreateMap();
void ScrollMap();
void VBLInterruptHandler(void);


// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    hel_PadInit();
    
    // initialize map system
    hel_MapInit(g_MapSystemBuffer);
    
    ham_SetBgMode(0);
    
    // Load Tileset into VideoRAM
    ham_bg[1].ti = ham_InitTileSet(ResData(RES_RACETRACK_RAW), RES_RACETRACK_RAW_SIZE16, 1, TRUE);

    // Create map
    CreateMap();
    
    // Load Background Palette
    hel_PalBgLoad256(ResData(RES_RACETRACK_PAL));

    // Turn on Background
    ham_InitBg(1, TRUE, 0, FALSE);

    // Start VBL Interrupt
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);

    // Infinite loop
    for(;;)
    {
        hel_PadCapture();
        ScrollMap();
        
        hel_SwiVBlankIntrWait();
    }

    return 0;
}

// -----------------------------------------------------------------------------
// Create the map
// -----------------------------------------------------------------------------
void CreateMap()
{
  TMapCreateIndirectDesc	desc;

  desc.BgNo = 1; // use background 1

  // Set map size
  desc.Size.X = 128; // 128 tiles wide
  desc.Size.Y = 128; // 128 tiles high
  desc.Size.Flags = MAP_FLAGS_SIZE0; // Use the smallest hardware map size

  // Pointer to actual map data
  desc.pData = ResData(RES_RACETRACK_MAP);
  desc.DataTypeSize = sizeof(u16); // size of one element in pData

  // Set the flags!
  desc.Flags = MAP_FLAGS_DEFAULT;

  // Set initial position
  desc.Position.X = FIXED_FROMINT(184); // move 184 pixels to the right
  desc.Position.Y = FIXED_FROMINT(96);  // move 96 pixels downwards

  // Set the map boundaries (in pixels)
  desc.Bounds.Left = FIXED_FROMINT(0);
  desc.Bounds.Top = FIXED_FROMINT(0);
  desc.Bounds.Right = FIXED_FROMINT(desc.Size.X*8);
  desc.Bounds.Bottom = FIXED_FROMINT(desc.Size.Y*8);
  desc.Bounds.Flags = MAP_BOUNDS_DEFAULT;

  // Finally create the map
  hel_MapCreateIndirect(&desc);
}

// -----------------------------------------------------------------------------
// Queries the Pad and scrolls the map depending on the pad input
// -----------------------------------------------------------------------------
void ScrollMap()
{
   #define SCROLL_SPEED_X     FIXED_FROMFLOAT(2.0f)
   #define SCROLL_SPEED_Y     FIXED_FROMFLOAT(2.0f)

   TPointSfp32 ScrollBy={0, 0};

   // Query Pad to see in what direction
   // the map should be scrolled
   if(hel_PadQuery()->Held.Left)
   {
      ScrollBy.X -= SCROLL_SPEED_X;
   }
   else
   if(hel_PadQuery()->Held.Right)
   {
      ScrollBy.X += SCROLL_SPEED_X;
   }

   if(hel_PadQuery()->Held.Down)
   {
      ScrollBy.Y += SCROLL_SPEED_Y;
   }
   else
   if(hel_PadQuery()->Held.Up)
   {
      ScrollBy.Y -= SCROLL_SPEED_Y;
   }

   // Now Scroll the map!
   hel_MapScroll(1, ScrollBy.X, ScrollBy.Y);
}

// -----------------------------------------------------------------------------
// VBL Interrupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler()
{
	hel_IntrAcknowledge(INT_TYPE_VBL);
}

