// -----------------------------------------------------------------------------
//
// Author(s):     Peter Schraut (www.console-dev.de)
//                Original code and idea by Andrzej Czyz (www.blindfold.z.pl)
//
// Last Modified: 18th April 2006
//
// Purpose:       Demo shows how to use hel/tools/win32/paletti.exe
//
// Notes:
//                The approach on how to create a nice gradient background with
//                only wasting ONE palette entry is actually taken from Andrzej Czyz
//                Agony Techdemo for the Gameboy Advance.
//                Andrzej's website can be reached at www.blindfold.z.pl
//
// -----------------------------------------------------------------------------
#include <hel2.h>
#include "ResourceData.h"

// -----------------------------------------------------------------------------
// Prototypes
// -----------------------------------------------------------------------------
void HBLInterruptHandler(void);
void VBLInterruptHandler(void);

// -----------------------------------------------------------------------------
// Program Entry Point
// -----------------------------------------------------------------------------
int main(void)
{
    ham_Init();
    hel_SysSetPrefetch(TRUE);
    
    // Set entire background
    // to palette entry 0
    ham_SetBgMode(4);
    ham_ClearBackBuffer(0);
    ham_FlipBGBuffer();

    // Start interrupts
    hel_IntrStartHandler(INT_TYPE_HBL, (void*)HBLInterruptHandler);
    hel_IntrStartHandler(INT_TYPE_VBL, (void*)VBLInterruptHandler);
    
    // Infinite loop to keep
    // the application alive
    for(;;)
    {
        hel_SwiVBlankIntrWait();
    }

    return 0;
}

// -----------------------------------------------------------------------------
// HBL Interupt Handler
// -----------------------------------------------------------------------------
void HBLInterruptHandler(void)
{
    // Here is the entire code for doing the gradient. We basically use the
    // F_VCNT_CURRENT_SCANLINE macro to get the index of the current scanline
    // and use this value to access our gradient palette (see gfx/gradient.bmp)
    // then we just set this value at palette entry 0 in Vram.
    ACCESS_16(MEM_PAL_BG_COL_256(0)) = ResData16(RES_GRADIENT_PAL)[F_VCNT_CURRENT_SCANLINE];
}

// -----------------------------------------------------------------------------
// VBL Interupt Handler
// -----------------------------------------------------------------------------
void VBLInterruptHandler(void)
{
    hel_IntrAcknowledge(INT_TYPE_VBL);
}

